/*
 * JMainFrame.java
 *
 * Created on 16. November 2006, 22:43
 *
 *  Copyright (C) 16. November 2006  <reiner>
 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */


package jpasswordgenerator;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.border.SoftBevelBorder;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SpinnerNumberModel;
    
import javax.swing.UIManager.LookAndFeelInfo;
import javax.swing.UIManager;

// our shared package
import shared.classpathhelper.JClassPathHelper;
import shared.browserlaunch.JBrowserLaunch;
import shared.files.JPathHelper;
import shared.jinfohelper.JInfoHelper;
import shared.layout.JGridBagPanel;
import shared.numberspinner.JNumberSpinner;
import shared.popupmenu.JMenuItemHelper;
import shared.popupmenu.JStatusMenu;
import shared.popupmenu.JStatusMenuItem;
import shared.popupmenu.JStatusRadioButtonMenuItem;
import shared.statusbar.JStatusBar;
import shared.datetimedisplay.JDateTimeDisplay;

/**
 *
 * @author reiner
 */

public class JMainFrame  extends JFrame
{
    private JCheckBox[] m_setCheckBox = new JCheckBox[Main.m_numberOfSets];
    private JTextField[] m_setTextField = new JTextField[Main.m_numberOfSets];
    private JButton[] m_resetButton = new JButton[Main.m_numberOfSets];
    private JNumberSpinner m_spinnerMin;
    private JNumberSpinner m_spinnerMax;
    private JTextField m_pass;
    private JMenuBar m_menuBar;
    private JStatusBar m_statusBar;
    private JDateTimeDisplay m_dateTimeDisplay;
    private static final String[] m_homepage = {"http://jpasswordgen.sourceforge.net/"};
    
    /** Creates a new instance of JMainFrame */
    public JMainFrame()
    {
        super("");
        setIconImage(Main.m_icon16.getImage());
        setTitle(Main.getString("caption"));

        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent ev)
            {
                doSaveSettings();
            }
        });
        m_menuBar = new JMenuBar();
        setJMenuBar(m_menuBar);

        m_statusBar = new JStatusBar(Main.getString("welcome"));
        m_dateTimeDisplay = new JDateTimeDisplay();
        m_dateTimeDisplay.setTimeFormat(JDateTimeDisplay.DATETIMEDISPLAY_TIME_MEDIUM);
        m_dateTimeDisplay.setDateFormat(JDateTimeDisplay.DATETIMEDISPLAY_DATE_SHORT);
        m_dateTimeDisplay.setSeparator(" - ");

        JMenuItemHelper menuItemHelper;
        // extra menu
        JStatusMenu menu = new JStatusMenu(Main.getString("menu_extra"), m_statusBar);

        // Look and Feel
        final JMenu lfMenu = new JMenu(Main.getString("menu_lookandfeel"));
        LookAndFeelInfo[] lfArray = UIManager.getInstalledLookAndFeels();
        JStatusRadioButtonMenuItem radioButtonMenuItem;
        int count = 0;
        for (LookAndFeelInfo lfInfo : lfArray)
        {
            radioButtonMenuItem = new JStatusRadioButtonMenuItem(lfInfo.getName(), String.format(Main.getString("statusbar_menu_extra_lf_help"), lfInfo.getName()), m_statusBar);
            radioButtonMenuItem.setActionCommand(lfInfo.getClassName());
            radioButtonMenuItem.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                    try
                    {
                        Main.m_settings.setLookAndFeel(ev.getActionCommand());
                        int i, anzahl = lfMenu.getItemCount();
                        for (i=0; i< anzahl; i++)
                        {
                            JStatusRadioButtonMenuItem menuItem = (JStatusRadioButtonMenuItem)lfMenu.getItem(i);
                            menuItem.setSelected(menuItem.getActionCommand().equals(ev.getActionCommand()));
                        }
                    }
                    catch(Exception ex)
                    {
                    }

                    // Restart
                    if (JOptionPane.showConfirmDialog(Main.m_mainFrame, Main.m_settings.getResBundle().getString("lookandfeel_restart_ask").replace("\\n", "\n"), Main.getMessageBoxCaption(), JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE) == JOptionPane.YES_OPTION)
                    {
                        try
                        {
                            String path = JClassPathHelper.getClassPath(this, true);
                            if (path != null)
                            {
                                doSaveSettings();
                                setVisible(false);
                                path = JPathHelper.addSeparator(path);
                                path += "jstart.jar";
                                String[] cmdArray = {JInfoHelper.getJavaExecutablePath(), "-jar", path};
                                if (Main.m_settings.isDebug())
                                {
                                    System.err.println(cmdArray[0]);
                                    System.err.println(path);
                                }
                                Runtime.getRuntime().exec(cmdArray);
                                System.exit(0);
                            }
                        }
                        catch(Exception ex)
                        {
                            setVisible(true);
                            System.err.println(ex);
                        }
                    }
                }
            });
            radioButtonMenuItem.setSelected(Main.m_settings.getLookAndFeel().equals(lfInfo.getClassName()));
            lfMenu.add(radioButtonMenuItem);
        }
        menu.add(lfMenu);
        m_menuBar.add(menu);

        // help menu
        menu = new JStatusMenu(Main.getString("menu_help"), m_statusBar);

        JStatusMenuItem menuItem = new JStatusMenuItem(Main.getString("menu_help_help"), Main.getString("statusbar_menu_help_help_help"), m_statusBar);
        menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
        menu.add(menuItem);
        menuItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                JBrowser browser = new JBrowser(Main.m_mainFrame, getClass().getResource(Main.getString("helpfile")));
                browser.setVisible(true);
            }
        });

        menuItem = new JStatusMenuItem(Main.getString("menu_help_about"), Main.getString("statusbar_menu_help_about_help"), m_statusBar);
        menu.add(menuItem);
        menuItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.append(Main.m_versionString);
                strBuilder.append("\n\n");
                strBuilder.append("Homepage:\n");
                strBuilder.append(m_homepage[0]);
                strBuilder.append("\n\n");
                strBuilder.append(JInfoHelper.getJavaInfo(Main.m_settings.isDebug() ? 2 : 1));
                if (Main.m_settings.isDebug())
                    strBuilder.append("\n\nMemory: (total, free, max): " + Runtime.getRuntime().totalMemory() + ", " + Runtime.getRuntime().freeMemory() + ", " + Runtime.getRuntime().maxMemory());
                JOptionPane.showMessageDialog(Main.m_mainFrame, strBuilder.toString(), Main.getMessageBoxCaption(), JOptionPane.PLAIN_MESSAGE, Main.m_icon32);
            }
        });

        menu.addSeparator();

        menuItem = new JStatusMenuItem(Main.getString("menu_help_web"), Main.getString("statusbar_menu_help_web_help"), m_statusBar);
        menu.add(menuItem);
        menuItem.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                if (!JBrowserLaunch.openURL(m_homepage[0]))
                {
                    JOptionPane.showMessageDialog(Main.m_mainFrame, String.format(Main.getString("msg_browserlaunch_failed"), m_homepage[0], m_homepage[1]), Main.getMessageBoxCaption(), JOptionPane.PLAIN_MESSAGE, Main.m_icon32);
                }
            }
        });
        m_menuBar.add(menu);

        Container cp = getContentPane();
        cp.setLayout(new BorderLayout());
        JGridBagPanel panel = new JGridBagPanel();
        Dimension txtDim = new Dimension(230, 20);
        int row = 0;
        int col = 0;
        for (int i=0; i<Main.m_numberOfSets; i++)
        {
            m_setCheckBox[i] = new JCheckBox(Main.getString(String.format("setname_%1$d", i+1)), Main.m_settings.isSet(i));
            m_setCheckBox[i].setToolTipText(Main.getString(String.format("setname_tooltip_%1$d", i+1)));
            final int j = i;
            m_setCheckBox[i].addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                    doSetSetState(j, ((JCheckBox)ev.getSource()).isSelected());
                }
            });
            panel.addComponent(m_setCheckBox[i], col, row, GridBagConstraints.WEST, new Insets(2, 0, 0, 0));
            m_resetButton[i] = new JButton(Main.getString(String.format("reset", i+1)));
            m_resetButton[i].setToolTipText(Main.getString(String.format("reset_tooltip_%1$d", i+1)));
            Dimension dimButton = m_resetButton[i].getPreferredSize();
            m_resetButton[i].setDefaultCapable(false);
            m_resetButton[i].setMargin(new Insets(0, 0, 0, 0));
            m_resetButton[i].setMinimumSize(new Dimension(45, 20));

            m_resetButton[i].addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent ev)
                {
                    m_setTextField[j].setText(Main.m_settings.getDefSet(j));
                }
            });
            panel.addComponent(m_resetButton[i], col, row, GridBagConstraints.EAST, new Insets(2, 0, 0, 7));

            row++;
            m_setTextField[i] = new JTextField(Main.m_settings.getSet(i), 35);
            m_setTextField[i].setMinimumSize(txtDim);
            m_setTextField[i].setEnabled(Main.m_settings.isSet(i));
            m_setTextField[i].setToolTipText(Main.getString(String.format("set_tooltip_%1$d", i+1)));
            panel.addComponent(m_setTextField[i], col, row, GridBagConstraints.WEST, new Insets(3, 0, 15, 7));
            row++;
            if (i == 2)
            {
                col++;
                row = 0;
            }
            doSetSetState(i, Main.m_settings.isSet(i));
        }

        row++;
        panel.addComponent(new JLabel(Main.getString("min")), 0, row, GridBagConstraints.WEST, new Insets(0, 0, 0, 0));
        panel.addComponent(new JLabel(Main.getString("max")), 1, row, GridBagConstraints.WEST, new Insets(0, 0, 0, 0));

        row++;
        m_spinnerMin = new JNumberSpinner(new SpinnerNumberModel(Main.m_settings.getMin(), 0, 30, 1));
        m_spinnerMin.setToolTipText(Main.getString("tooltip_min"));
        panel.addComponent(m_spinnerMin, 0, row, GridBagConstraints.WEST, new Insets(0, 0, 15, 7));
        m_spinnerMax = new JNumberSpinner(new SpinnerNumberModel(Main.m_settings.getMax(), 0, 30, 1));
        m_spinnerMax.setToolTipText(Main.getString("tooltip_max"));
        panel.addComponent(m_spinnerMax, 1, row, GridBagConstraints.WEST, new Insets(0, 0, 15, 7));

        JButton button = new JButton("Create");
        panel.addComponent(button, 1, row, GridBagConstraints.EAST, new Insets(3, 0, 15, 7));
        getRootPane().setDefaultButton(button);
        button.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                setGuiDataToSettings();
                m_pass.setText(Main.doCreatePassword());
                m_pass.selectAll();
                m_pass.copy();
                m_pass.select(0, 0);
            }
        });
        button.setToolTipText(Main.getString("tooltip_create"));

        row++;
        m_pass = new JTextField("");
        m_pass.setFont(m_pass.getFont().deriveFont(Font.BOLD, Main.m_settings.getPassFontSize()));
        m_pass.setHorizontalAlignment(JTextField.CENTER);
        m_pass.setMinimumSize(new Dimension(txtDim.width*2 + 7, 50));
        m_pass.setEditable(false);
        m_pass.setBorder(new SoftBevelBorder(SoftBevelBorder.LOWERED));
        m_pass.setToolTipText(Main.getString("tooltip_pass"));
        panel.addComponent(m_pass, 0, row, GridBagConstraints.WEST, new Insets(3, 0, 15, 7), 2, 1);

        cp.add(panel, BorderLayout.CENTER);

        JPanel south = new JPanel();
        south.setLayout(new BorderLayout(0, 0));
        south.add(m_statusBar, BorderLayout.CENTER);
        m_dateTimeDisplay.setText("XXX");
        south.add(m_dateTimeDisplay, BorderLayout.EAST);
        cp.add(south, BorderLayout.SOUTH);

        setPreferredSize(new Dimension(500, 400));
        setResizable(false);
        pack();
    }
	
    private void doSetSetState(int index, boolean bEnabled)
    {
        m_setTextField[index].setEnabled(bEnabled);
        m_resetButton[index].setEnabled(bEnabled);
    }
    
    private void setGuiDataToSettings()
    {   
        for (int i=0; i<Main.m_numberOfSets; i++)
        {
            Main.m_settings.setIsSet(i, m_setCheckBox[i].isSelected());
            Main.m_settings.setSet(i, m_setTextField[i].getText());
        }
        Main.m_settings.setMin((Integer)m_spinnerMin.getValue());
        Main.m_settings.setMax((Integer)m_spinnerMax.getValue());
    }

    private void doSaveSettings()
    {   
        setGuiDataToSettings();
        Main.m_settings.save(JPathHelper.addSeparator(System.getProperty("user.home")) + JSettings.SETTINGS_FILE);
    }
}
