/*
 * JSettings.java
 *
 * Created on 16. November 2006, 22:43
 *
 *  Copyright (C) 16. November 2006  <reiner>
 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */


package jpasswordgenerator;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;

import java.util.Properties;

import shared.settings.JSettingsHelper;

/**
 *
 * @author reiner
 */

public class JSettings extends JSettingsHelper
{
    public static final String SETTINGS_FILE = "jpasswordgenerator.properties";
    public static final String COMMENT = "JPasswordGenerator settings";
    
    private static final String[] SET = {"SET1", "SET2", "SET3", "SET4", "SET5", "SET6"};
    private static final String[] ISSET = {"ISSET1", "ISSET2", "ISSET3", "ISSET4", "ISSET5", "ISSET6"};
    private static final String MIN = "Min";
    private static final String MAX = "Max";
    private static final String PASSFONTSIZE = "PassFontSize";
 
    private static final String[] DEF_SET = {"ABCDEFGHIJKLMNOPQRSTUVWXYZ", "abcdefghijklmnopqrstuvwxyz", "0123456789", "/\\#*+-_.:,;!\"$%&(){[]}?@~|<>", "", ""};
    private static final String[] DEF_ISSET = {"true", "true", "true", "true", "false", "false"};
    private static final String DEF_MIN = "8";
    private static final String DEF_MAX = "8";
    private static final String DEF_PASSFONTSIZE = "36";

    private String[] m_set = new String[Main.m_numberOfSets];
    private boolean[] m_bSet = new boolean[Main.m_numberOfSets];
    private int m_min;
    private int m_max;
    private int m_passFontSize;
    
    /** Creates a new instance of JSettings */
    public JSettings(boolean bInitSwing)
    {
    	super(bInitSwing);
    }
    
    protected String getComment()
    {
        return COMMENT;
    }
    
    public String getDefSet(int index)
    {
        assert index >= 0 && index < Main.m_numberOfSets;
        return DEF_SET[index];
    }

    public String getSet(int index)
    {
        assert index >= 0 && index < Main.m_numberOfSets;
        return m_set[index];
    }

    public void setSet(int index, String set)
    {
    	assert index >= 0 && index < Main.m_numberOfSets;
        m_set[index] = set;
    }
    
    public boolean isSet(int index)
    {
        assert index >= 0 && index < Main.m_numberOfSets;
        return m_bSet[index];
    }
    
    public void setIsSet(int index, boolean isSet)
    {
        assert index >= 0 && index < Main.m_numberOfSets;
        m_bSet[index] = isSet;
    }

    public int getMin()
    {
    	return m_min;
    }
    
    public void setMin(int min)
    {
        m_min = min;
    }

    public int getMax()
    {
    	return m_max;
    }
    
    public void setMax(int max)
    {
        m_max = max;
    }

    public int getPassFontSize()
    {
    	return m_passFontSize;
    }
    
    public void setPassFontSize(int passFontSize)
    {
    	m_passFontSize = passFontSize;
    }

    protected void doLoad(Properties prop)
    {
        for (int i=0; i<Main.m_numberOfSets; i++)
        {
            m_set[i] = prop.getProperty(SET[i], DEF_SET[i]);
            m_bSet[i] = new Boolean(prop.getProperty(ISSET[i], DEF_ISSET[i]));
        }
        m_min = new Integer(prop.getProperty(MIN, DEF_MIN));
        m_max = new Integer(prop.getProperty(MAX, DEF_MAX));
        m_passFontSize = new Integer(prop.getProperty(PASSFONTSIZE, DEF_PASSFONTSIZE));
    }
    
    protected void doSave(Properties prop)
    {
        for (int i=0; i<Main.m_numberOfSets; i++)
        {
            prop.setProperty(SET[i], m_set[i]);
            prop.setProperty(ISSET[i], Boolean.toString(m_bSet[i]));
        }
        prop.setProperty(MIN, Integer.toString(m_min));
        prop.setProperty(MAX, Integer.toString(m_max));
        prop.setProperty(PASSFONTSIZE, Integer.toString(m_passFontSize));
    }
}
