/*
 * Main.java
 *
 * Created on 16. November 2006, 22:43
 *
 *  Copyright (C) 16. November 2006  <reiner>
 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package jpasswordgenerator;

import javax.swing.ImageIcon;
import javax.swing.JFrame;

import java.util.ArrayList;
import java.util.Random;

import java.security.SecureRandom;
import java.security.NoSuchAlgorithmException;
    
// our shared package
import shared.files.JPathHelper;

/**
 *
 * @author reiner
 */
public class Main
{
    public static final String m_version = "1.2";
    public final static String m_versionString = "Author: Reiner Pr\u00F6ls\nVersion: " + m_version + "\nCopyright \u00A9 2006-2010 by Reiner Pr\u00F6ls\nLicense: GPL";

    public static JMainFrame m_mainFrame;
    public static JSettings m_settings;
    public final static int m_numberOfSets = 6;
    public static ImageIcon m_icon16;
    public static ImageIcon m_icon32;

    /** Creates a new instance of Main */
    public Main()
    {
    }
    
    public static String getMessageBoxCaption()
    {
        String str = null;
        try
        {
            str = m_settings.getResBundle().getString("caption");
        }
        catch(Exception ex)
        {
        }
        if (str == null) str = new String("JPasswordGenerator");
        return str;
    }
    
    public static String getString(String key)
    {
    	return m_settings.getResBundleString(key);
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {
        final String arguments[] = args.clone();

        // Commandline handling
        // TO DO
        boolean bNoGui = false;
        boolean bDebug = false;
        int min = 0;
        int max = 0;
        String[] s = new String[m_numberOfSets];
        if (args.length > 0)
        {
            String str, tag;
            int i = 0;
            while (i < args.length)
            {
            str = args[i++];
            if (str.charAt(0) == '-' || str.charAt(0) == '/')
            {
                tag = str.substring(1);
                if (tag.equals("ng")) // no gui
                bNoGui = true;
                else if (tag.equals("d")) // debug
                {
                    bDebug = true;
                    System.err.println("Debug");
                }
                else if (tag.equals("v")) // version
                {
                    System.out.println(m_versionString);
                    System.exit(0);
                }
                else if (tag.equals("m")) // minimum password length
                {
                    if (i < args.length)
                        min = Integer.parseInt(args[i++]);
                }
                else if (tag.equals("M")) // maximum password length
                {
                    if (i < args.length)
                        max = Integer.parseInt(args[i++]);
                }
                else
                {
                    for (int j=0; j<m_numberOfSets; j++)
                    {
                        if (tag.equals(String.format("s%1$d", j+1))) // setj
                        {
                            if (i < args.length)
                                s[j] = args[i++];
                            break;
                        }
                    }
                }
            }
            }
        }

        // Load settings 
        m_settings = new JSettings(!bNoGui);

        if (!bNoGui)
        {
            m_icon16 = new ImageIcon(Main.class.getResource("icon16x16.png"));
            m_icon32 = new ImageIcon(Main.class.getResource("icon32x32.png"));
            m_settings.load(JPathHelper.addSeparator(System.getProperty("user.home")) + m_settings.SETTINGS_FILE, m_version);
            javax.swing.SwingUtilities.invokeLater(
            new Runnable()
            {
                public void run()
                {
                    initGui(arguments);
                }
            });
        }
        else
        {
            if (min > 0)
                m_settings.setMin(min);
            if (max > 0)
                m_settings.setMax(max);
            if (bDebug)
                m_settings.setIsDebug(bDebug);
            for (int j=0; j<m_numberOfSets; j++)
            {
                if (s[j] != null)
                {
                    m_settings.setIsSet(j, true);
                    m_settings.setSet(j, s[j]);
                }
                else m_settings.setIsSet(j, false);
            }
            System.out.println(doCreatePassword());
            while(true)
            {
            ;
            }
        }
    }
    
    private static void initGui(String[] args)
    {
        if (!m_settings.init("JPasswordGeneratorLang"))
            System.exit(1);

        m_mainFrame = new JMainFrame();
        m_mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        m_mainFrame.setVisible(true);
    }
    
    /**
     * this is the core function
     * we have placed it in the Main calss because will will later add a commandline interface without GUI
     */
    
    public static String doCreatePassword()
    {
        int min = m_settings.getMin();
        int max = m_settings.getMax();
        if (max < min)
        {
            int h = max;
            max = min;
            min = h;
        }
        assert min <= max;
        int count = 0;
        ArrayList<Character> buf = new ArrayList<Character>();
        for (int i=0; i<m_numberOfSets; i++)
        {
            if (m_settings.isSet(i))
            {
                char[] chArray = m_settings.getSet(i).toCharArray();
                for (int j=0; j<chArray.length; j++)
                    buf.add(chArray[j]);
            }
        }
        if (m_settings.isDebug())
        {
            StringBuilder strBuilder = new StringBuilder();
            for (Character ch : buf)
            strBuilder.append(ch);
            System.err.println(strBuilder);
        }

        // Create a random number generator
        Random random = null;
        try
        {
            random = SecureRandom.getInstance("SHA1PRNG");
        }
        catch (NoSuchAlgorithmException ex)
        {
            // using default Random class
            random = new Random();
        }

        int length = 0;
        if (max == min) length = min;
        else length = random.nextInt(max - min + 1) + min;
        if (m_settings.isDebug())
            System.err.println("Length: "+length);

        StringBuilder strBuilder = new StringBuilder();
        while(length-- > 0)
            strBuilder.append(buf.get(random.nextInt(buf.size())));
        return strBuilder.toString();
    }
}
