/*
 *  JXmlHelper.java
 *
 *  Created on 08.09.2009, 13:31:51
 *
 *  Copyright (C) 08.09.2009  reiner

 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */


/**
 *
 * @author reiner
 */

package shared.xml;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

public final class JXmlHelper
{
    private JXmlHelper()
    {}

    public static Node findNextNode(String name, NodeList list)
    {
        return findNextNode(name, list, null);
    }

    public static Node findNextNode(String name, NodeList list, JXmlNodeIndex index)
    {
        int anzahl = list.getLength(), i;
        for (i=index != null ? index.getIndex() : 0; i<anzahl; i++)
        {
            Node node = list.item(i);
            if (node.getNodeType() == Node.ELEMENT_NODE)
            {
                if (node.getNodeName().equals(name))
                {
                    if (index != null)
                        index.setIndex(i+1);
                    return node;
                }
            }
        }
        return null;
    }

    public static String getNodeText(Node node)
    {
        return (node != null && (node = node.getFirstChild()) != null && node.getNodeType() == Node.TEXT_NODE) ? node.getNodeValue() : "";
    }

    public static String getNodeTextStrict(Node node)
    {
        return (node != null && (node = node.getFirstChild()) != null && node.getNodeType() == Node.TEXT_NODE) ? node.getNodeValue() : null;
    }

    public static String getAttribute(Node node, String key)
    {
        NamedNodeMap map = node.getAttributes();
        return (map != null ? getNodeText(map.getNamedItem(key)) : null);
    }

    public static Text createNewLineTab(Document doc, int tabCount)
    {
        String nlStr = "\n";
        String tabStr = "\t";
        StringBuilder strBuilder = new StringBuilder();
        strBuilder.append(nlStr);
        while (tabCount-- > 0)
            strBuilder.append(tabStr);
        return doc.createTextNode(strBuilder.toString());
    }
}
